// Uncomment this to run O(n^2) checks whether the contours don't
// touch and have matching inner/outer heights.
//#define RUN_CHECKS 1

#include <iostream>
#include <sstream>
#include <string>
#include <vector>
#include <set>
#include <algorithm>
#include <cmath>
#include <cstdio>

using namespace std;

typedef long long coord;

const double EPS = 1E-8;

const int HIT_INTERSECTS = 1;

const int LINE    = 0;
const int SEGMENT = 1;

class point {
public:
	coord x, y;

	point() { x = y = 0; }
	point(coord x1, coord y1) { x = x1; y = y1; }

	friend ostream &operator <<(ostream &os, point p);

	string str()
	{
		ostringstream res;
		res << *this;
		return res.str();
	}

	point operator +=(point a) { x += a.x; y += a.y; return *this; }
	point operator -=(point a) { x -= a.x; y -= a.y; return *this; }
	point operator *=(coord a) { x *= a;   y *= a;   return *this; }
	point operator /=(coord a) { x /= a;   y /= a;   return *this; }
	int   operator ==(point a) { return x==a.x && y==a.y; }
	int   operator !=(point a) { return x!=a.x || y!=a.y; }
};

point operator *(coord x, point a) { return a *= x; }
point operator *(point a, coord x) { return a *= x; }
point operator /(point a, coord x) { return a /= x; }
point operator +(point a, point b) { return a += b; }
point operator -(point a, point b) { return a -= b; }
coord operator *(point a, point b) { return a.x*b.x + a.y*b.y; }

coord cross(point a, point b) { return a.x*b.y - a.y*b.x; }
double length(point a) { return sqrt((double)(a*a)); }
point perp(point a) { return point(-a.y,a.x); }

class line {
public:
	point p1, p2;

	line() { p1 = p2 = point(); }
	line(const point &q1, const point &q2) { p1 = q1; p2 = q2; }

	friend ostream &operator <<(ostream &os, line l);

	string str()
	{
		ostringstream res;
		res << *this;
		return res.str();
	}
};

ostream &operator <<(ostream &os, point p)  { return os << '(' << p.x << ',' << p.y << ')'; }
ostream &operator <<(ostream &os, line l)   { return os << l.p1 << '-' << l.p2; }

double dist(point a, line b, int segment = 0)
{
	point db = b.p2 - b.p1;
	point dbperp = perp(db);

	if ( segment && ((b.p1-a)*db)*((b.p2-a)*db)>0 ) {
		return min(length(b.p1-a),length(b.p2-a));
	}

	return (a - b.p1)*dbperp / length(dbperp);
}

struct dpoint {
	double x,y;
};

int intersect(line a, line b, dpoint *s = NULL, int segment = 0)
{
	// directions of lines
	point da = a.p2 - a.p1;
	point db = b.p2 - b.p1;
	double eps;

	if ( length(da)<EPS || length(db)<EPS ) return 0;

	// false if lines are parallel
	if ( fabs(cross(da,db)/(length(da)*length(db)))<EPS ) return 0;

	// calc intersection point (as floating point!)
	if ( s!=NULL ) {
		double c = (double)cross(da,a.p1-b.p1)/cross(da,db);
		s->x = b.p1.x + c*db.x;
		s->y = b.p1.y + c*db.y;
	}

	// return false if line segments not really cross
	if ( segment ) {
		eps = HIT_INTERSECTS ? +EPS : -EPS;
		if ( cross(da,b.p1-a.p1)*cross(da,b.p2-a.p1)>eps ) return 0;
		if ( cross(db,a.p1-b.p1)*cross(db,a.p2-b.p1)>eps ) return 0;
	}

	return 1;
}

struct contour {
	int hout, hin;
	vector<point> poly;

	point& operator [](size_t i) { return poly[i]; }
	const point& operator [](size_t i) const { return poly[i]; }
	size_t size() const { return poly.size(); }
};

// Check if contour a is inside b (i.e. smaller):
int operator <(const contour &a, const contour &b)
{
	line linfty(a[0],a[0]+point(1,2000013));
	int nsect = 0;

	for(size_t j=0; j<b.size(); j++) {
		line l(b[j],b[(j+1)%b.size()]);
		if ( intersect(line(b[j],b[(j+1)%b.size()]),linfty,NULL,SEGMENT) ) nsect++;
	}

	return nsect%2==1;
}

vector<contour> contours;

// Return contour that point is directly contained in (i.e. not inside
// a smaller contour), return -1 if not inside any contour.
int inside(point x, int skip = -1, bool outermost = false)
{
	vector<set<double>> sect(contours.size());

	line linfty(x,x+point(1,2000013));
	for(size_t i=0; i<contours.size(); i++) {
		if ( (int)i==skip ) continue;
		for(size_t j=0; j<contours[i].size(); j++) {
			line l(contours[i][j],contours[i][(j+1)%contours[i].size()]);
			dpoint s;
			if ( intersect(linfty,l,&s,SEGMENT) ) sect[i].insert(hypot(s.x-x.x,s.y-x.y));
		}
	}

	int res = -1;

	for(size_t i=0; i<contours.size(); i++) {
		if ( sect[i].size()%2==1 ) {
			if ( res==-1 ) {
				res = (int)i;
			} else if ( (!outermost && *sect[i].begin() <*sect[res].begin() ) ||
			            ( outermost && *sect[i].rbegin()>*sect[res].rbegin()) ) {
				res = (int)i;
			}
		}
	}

	return res;
}

int main()
{
	size_t n;
	cin >> n;

	contours.resize(n);

	for(size_t i=0; i<n; i++) {
		int m;
		cin >> contours[i].hout >> contours[i].hin >> m;
		contours[i].poly.resize(m);
		for(int j=0; j<m; j++) cin >> contours[i][j].x >> contours[i][j].y;
	}

#ifdef RUN_CHECKS
	// Check that no contour lines cross
	for(size_t i1=0; i1<n; i1++) {
		for(size_t j1=0; j1<contours[i1].size(); j1++) {

			for(size_t j2=j1+2; j2<contours[i1].size(); j2++) {
				if ( (j2+1)%contours[i1].size()==j1 ) continue;
				if ( intersect(line(contours[i1][j1],contours[i1][j1+1]),
				               line(contours[i1][j2],contours[i1][(j2+1)%contours[i1].size()]),
				               NULL,SEGMENT) ) {
					cerr << "Contour " << i1 << " edge " << j1
						 << " self-intersects edge " << j2 << ".\n";
					return 1;
				}
			}
			for(size_t i2=i1+1; i2<n; i2++) {
				for(size_t j2=0; j2<contours[i2].size(); j2++) {
					if ( intersect(line(contours[i1][j1],contours[i1][(j1+1)%contours[i1].size()]),
					               line(contours[i2][j2],contours[i2][(j2+1)%contours[i2].size()]),
					               NULL,SEGMENT) ) {
					cerr << "Contour " << i1 << " edge " << j1
						 << " intersects contour " << i2 << " edge " << j2 << ".\n";
					return 1;
					}
				}
			}
		}
	}

	// Check that inner/outer contour heights match
	int hout = -1;
	for(size_t i=0; i<n; i++) {
		int c = inside(contours[i][0],i);
		if ( c==-1 ) {
			if ( hout==-1 ) {
				hout = contours[i].hout;
			} else {
				if ( hout!=contours[i].hout ) {
					cerr << "Outermost height from contour " << i
						 << " does not match previously found: "
						 << contours[i].hout << " != " << hout << endl;
					return 1;
				}
			}
		} else {
			if ( contours[c].hin!=contours[i].hout ) {
				cerr << "Contours " << i << " is inside " << c
					 << ", but out/in heights differ: "
					 << contours[i].hout << " != " << contours[c].hin << endl;
				return 1;
			}
		}
	}
#endif // RUN_CHECKS

	double res = 1E99;
	point orig(0,0);

	// First check all possible points on contours closest to the origin.
	for(size_t i=0; i<n; i++) {
		for(size_t j=0; j<contours[i].size(); j++) {
			line l(contours[i][j],contours[i][(j+1)%contours[i].size()]);
			double d = dist(orig,l,SEGMENT);
			double h = min(contours[i].hin,contours[i].hout);
			res = min(res,hypot(d,h));
		}
	}

	// Secondly, check which contour the origin is in and see if a
	// vertical drill is optimal.
	int cwell = inside(orig);
	if ( cwell!=-1 ) {
		res = min(res,(double)contours[cwell].hin);
	} else {
		int c = inside(contours[0][0],0,true);
		if ( c==-1 ) c = 0;
		res = min(res,(double)contours[c].hout);
	}

	cout.precision(10);
	cout << res << endl;

	return 0;
}
