#include <iostream>
#include <sstream>
#include <string>
#include <vector>
#include <cmath>
#include <cstdio>

using namespace std;

typedef int coord;

const double EPS = 1E-8;

const int HIT_INTERSECTS = 1;

const int LINE    = 0;
const int SEGMENT = 1;

class point {
public:
	coord x, y;

	point() { x = y = 0; }
	point(coord x1, coord y1) { x = x1; y = y1; }

// BEGIN CUT HERE
	friend ostream &operator <<(ostream &os, point p);

	string str()
	{
		ostringstream res;
		res << *this;
		return res.str();
	}
// END CUT HERE

	point operator +=(point a) { x += a.x; y += a.y; return *this; }
	point operator -=(point a) { x -= a.x; y -= a.y; return *this; }
	point operator *=(coord a) { x *= a;   y *= a;   return *this; }
	point operator /=(coord a) { x /= a;   y /= a;   return *this; }
	int   operator ==(point a) { return x==a.x && y==a.y; }
	int   operator !=(point a) { return x!=a.x || y!=a.y; }
};

point operator *(coord x, point a) { return a *= x; }
point operator *(point a, coord x) { return a *= x; }
point operator /(point a, coord x) { return a /= x; }
point operator +(point a, point b) { return a += b; }
point operator -(point a, point b) { return a -= b; }
coord operator *(point a, point b) { return a.x*b.x + a.y*b.y; }

coord cross(point a, point b) { return a.x*b.y - a.y*b.x; }
double length(point a) { return sqrt((double)(a*a)); }
point perp(point a) { return point(-a.y,a.x); }

class line {
public:
	point p1, p2;

	line() { p1 = p2 = point(); }
	line(point q1, point q2) { p1 = q1; p2 = q2; }

// BEGIN CUT HERE
	friend ostream &operator <<(ostream &os, line l);

	string str()
	{
		ostringstream res;
		res << *this;
		return res.str();
	}
// END CUT HERE

	line operator +=(point a) { p1 += a; p2 += a; return *this; }
	line operator -=(point a) { p1 -= a; p2 -= a; return *this; }
	int  operator ==(line a) { return (p1==a.p1 && p2==a.p2) || (p1==a.p2 && p2==a.p1); }
};

line operator +(line a, point b) { return a += b; }
line operator -(line a, point b) { return a -= b; }

// BEGIN CUT HERE
ostream &operator <<(ostream &os, point p)  { return os << '(' << p.x << ',' << p.y << ')'; }
ostream &operator <<(ostream &os, line l)   { return os << l.p1 << '-' << l.p2; }
// END CUT HERE

double dist(point a, line b, int segment = 0)
{
	point db = b.p2 - b.p1;
	point dbperp = perp(db);

	if ( segment && ((b.p1-a)*db)*((b.p2-a)*db)>0 ) {
		return min(length(b.p1-a),length(b.p2-a));
	}

	return (a - b.p1)*dbperp / length(dbperp);
}

int intersect(line a, line b, int segment = 0)
{
	// directions of lines
	point da = a.p2 - a.p1;
	point db = b.p2 - b.p1;
	double eps;

	if ( length(da)<EPS || length(db)<EPS ) return 0;

	// false if lines are parallel
	if ( fabs(cross(da,db)/(length(da)*length(db)))<EPS ) return 0;

	// return false if line segments not really cross
	if ( segment ) {
		eps = HIT_INTERSECTS ? +EPS : -EPS;
		if ( cross(da,b.p1-a.p1)*cross(da,b.p2-a.p1)/(da*da)>eps ) return 0;
		if ( cross(db,a.p1-b.p1)*cross(db,a.p2-b.p1)/(db*db)>eps ) return 0;
	}

	return 1;
}

vector<vector<point>> contour;
vector<pair<int,int>> height;

int main()
{
	int n;
	cin >> n;

	contour.resize(n);
	height.resize(n);

	for(int i=0; i<n; i++) {
		int m;
		cin >> height[i].first >> height[i].second >> m;
		contour[i].resize(m);
		for(int j=0; j<m; j++) cin >> contour[i][j].x >> contour[i][j].y;
	}

	double res = 1E99;
	point orig(0,0);
	line l(orig,point(1,1000013));
	vector<double> cdist(n,1E99);

	for(int i=0; i<n; i++) {
		for(int j=0; j<contour[i].size(); j++) {
			double d = dist(orig,line(contour[i][j],contour[i][(j+1)%contour[i].size()]),SEGMENT);
			double h = min(height[i].first,height[i].second);
			res = min(res,hypot(d,h));

			cdist[i] = min(cdist[i],d);
		}
	}

	// FIXME: this doesn't work when no contour contains the origin!
	int cbest = 0;
	for(int i=1; i<n; i++) if ( cdist[i]<cdist[cbest] ) cbest = i;
	res = min(res,(double)height[cbest].second);

	cout.precision(10);
	cout << res << endl;

	return 0;
}
