#include <iostream>
#include <sstream>
#include <vector>
#include <map>
#include <queue>
#include <cmath>
#include <cstdio>

using namespace std;


typedef long long coord;

const double EPS = 1E-8;

const int HIT_INTERSECTS = 0;

const int LINE    = 0;
const int SEGMENT = 1;

class point {
public:
	coord x, y;

	point() { x = y = 0; }
	point(coord x1, coord y1) { x = x1; y = y1; }

	friend ostream &operator <<(ostream &os, point p);

	string str()
	{
		ostringstream res;
		res << *this;
		return res.str();
	}

	point operator +=(point a) { x += a.x; y += a.y; return *this; }
	point operator -=(point a) { x -= a.x; y -= a.y; return *this; }
	point operator *=(coord a) { x *= a;   y *= a;   return *this; }
	point operator /=(coord a) { x /= a;   y /= a;   return *this; }
	int   operator ==(point a) { return x==a.x && y==a.y; }
	int   operator !=(point a) { return x!=a.x || y!=a.y; }
};

point operator *(coord x, point a) { return a *= x; }
point operator *(point a, coord x) { return a *= x; }
point operator /(point a, coord x) { return a /= x; }
point operator +(point a, point b) { return a += b; }
point operator -(point a, point b) { return a -= b; }
coord operator *(point a, point b) { return a.x*b.x + a.y*b.y; }

coord cross(point a, point b) { return a.x*b.y - a.y*b.x; }
double length(point a) { return sqrt((double)(a*a)); }
point perp(point a) { return point(-a.y,a.x); }

class line {
public:
	point p1, p2;

	line() { p1 = p2 = point(); }
	line(const point &q1, const point &q2) { p1 = q1; p2 = q2; }

	friend ostream &operator <<(ostream &os, line l);

	string str()
	{
		ostringstream res;
		res << *this;
		return res.str();
	}
};

ostream &operator <<(ostream &os, point p)  { return os << '(' << p.x << ',' << p.y << ')'; }
ostream &operator <<(ostream &os, line l)   { return os << l.p1 << '-' << l.p2; }

int intersect(line a, line b, int segment = 1)
{
	// directions of lines
	point da = a.p2 - a.p1;
	point db = b.p2 - b.p1;
	double eps;

	if ( length(da)<EPS || length(db)<EPS ) return 0;

	// false if lines are parallel
	if ( fabs(cross(da,db)/(length(da)*length(db)))<EPS ) return 0;

	// return false if line segments not really cross
	if ( segment ) {
		eps = HIT_INTERSECTS ? +EPS : -EPS;
		if ( cross(da,b.p1-a.p1)*cross(da,b.p2-a.p1)>eps ) return 0;
		if ( cross(db,a.p1-b.p1)*cross(db,a.p2-b.p1)>eps ) return 0;
	}

	return 1;
}

const int maxn = 12;
const int maxenergy = 100;

const point corner[4] = {point(1,1),point(-1,1),point(-1,-1),point(1,-1)};
const point base(1000013,0);

vector<point> lamp;
vector<int> energy;

int area(point a, point b, point c)
{
	int res = 0;

	for(size_t i=0; i<lamp.size(); i++) {
		int sign = cross(b-a,c-a)<0 ? -1 : 1;
		if ( intersect(line(base,lamp[i]),line(a,b))+
		     intersect(line(base,lamp[i]),line(b,c))+
		     intersect(line(base,lamp[i]),line(c,a))==1 ) res += sign*energy[i];
	}

	return res;
}

struct edge {
	int l1, l2; // from/to lamp
	int c1, c2; // and corner
	double len;
	int area; // Energy "area" covered by oriented triangle
	          // consisting of edge and base point.
};

vector<edge> edges;

struct quepos {
	int l, c, area;
	double d;
};

int operator <(quepos a, quepos b) { return a.d > b.d; }

int main()
{
	int n, etot = 0, epos = 0, eneg = 0;
	cin >> n;

	lamp.resize(n);
	energy.resize(n);
	for(int i=0; i<n; i++) {
		cin >> lamp[i].x >> lamp[i].y >> energy[i];
		etot += energy[i];
		if ( energy[i]>0 ) {
			epos += energy[i];
		} else {
			eneg += energy[i];
		}
	}

	if ( etot%2!=0 ) {
		cout << "IMPOSSIBLE\n";
		return 0;
	}

	edge e;
	for(e.l1=0; e.l1<n; e.l1++) {
		for(e.c1=0; e.c1<4; e.c1++) {
			point p1 = lamp[e.l1]+corner[e.c1];

			// First connect neighboring corners
			e.l2 = e.l1;
			e.c2 = (e.c1+1)%4;
			point p2 = lamp[e.l2]+corner[e.c2];
			e.len = 2;
			e.area = area(p1,p2,base);
			edges.push_back(e);

			for(e.l2=e.l1+1; e.l2<n; e.l2++) {
				for(e.c2=0; e.c2<4; e.c2++) {
					point p2 = lamp[e.l2]+corner[e.c2];
					line lc(p1,p2);

					// Check for lamps blocking this connection
					for(int l=0; l<n; l++) {
						for(int a=0; a<4; a++) {
							for(int b=a+1; b<4; b++) {
								line lb(lamp[l]+corner[a],lamp[l]+corner[b]);
								if ( intersect(lc,lb) ) goto nextline;
							}
						}
					}
					e.len = length(p2-p1);
					e.area = area(p1,p2,base);
					edges.push_back(e);
// 					fprintf(stderr,"edge %d,%d -> %d,%d, len = %6.2lf, area = %3d,  line = (%2d,%2d) - (%2d,%2d)\n",
// 					        e.l1,e.c1,e.l2,e.c2,e.len,e.area,lc.p1.x,lc.p1.y,lc.p2.x,lc.p2.y);

				  nextline:
					if ( 0==1 ) continue; // dummy statement
				}
			}
		}
	}

	map<pair<int,int>,double> dist; // (lamp*corner,area)

	quepos start;
	start.d = 0;
	start.area = 0;
	for(start.l=0; start.l<n; start.l++) {
		for(start.c=0; start.c<4; start.c++) {
			priority_queue<quepos> que;
			que.push(start);
//			dist[make_pair(4*start.l+start.c,start.area)] = 0;

			while ( !que.empty() ) {
				quepos curr = que.top(); que.pop();

				pair<int,int> pii(4*curr.l+curr.c,curr.area);
				if ( dist.count(pii) ) {
//					fprintf(stderr,"skipping %d,%d,%3d\n",curr.l,curr.c,curr.area);
					continue;
				}
				dist[pii] = curr.d;
//				fprintf(stderr,"dist[%d,%d,%3d] = %6.2lf\n",curr.l,curr.c,curr.area,curr.d);
				if ( curr.l==start.l && curr.c==start.c && 2*curr.area==etot ) {
					cout << curr.d << endl;
					return 0;
				}

				for(size_t i=0; i<edges.size(); i++) {
					quepos next;
					next.d = -1;
					if ( curr.l==edges[i].l1 && curr.c==edges[i].c1 ) {
						next.l = edges[i].l2;
						next.c = edges[i].c2;
						next.d = curr.d + edges[i].len;
						next.area = curr.area + edges[i].area;
					}
					if ( curr.l==edges[i].l2 && curr.c==edges[i].c2 ) {
						next.l = edges[i].l1;
						next.c = edges[i].c1;
						next.d = curr.d + edges[i].len;
						next.area = curr.area - edges[i].area;
					}
					if ( next.d>=0 && (next.area>=eneg && next.area<=epos) ) {
// 						fprintf(stderr,"visiting %d,%d,%3d at d = %6.2lf\n",
// 						        next.l,next.c,next.area,next.d);
						que.push(next);
					}
				}
			}
		}
	}

	cout << "IMPOSSIBLE\n";

	return 0;
}
